CounterMask

Struct CounterMask 

pub struct CounterMask<T = usize> { /* private fields */ }
Counter index mask structure in SBI function calls for the PMU extension 11.

Implementations

Source

impl<T> CounterMask<T>
where T: SbiRegister,

pub const IGNORE_MASK: T = T::FULL_MASK

Special value to ignore the mask, and consider all bits as set.

pub const fn from_mask_base( counter_idx_mask: T, counter_idx_base: T, ) -> CounterMask<T>

Construct a CounterMask from mask value and base counter index.

pub const fn ignore_mask(&self) -> T

Gets the special value for ignoring the mask parameter.

pub const fn into_inner(self) -> (T, T)

Returns mask and base parameters from the CounterMask.

Source

impl CounterMask

pub const fn has_bit(self, counter: usize) -> bool

Returns whether the CounterMask contains the provided counter.