pub struct CounterMask<T = usize> { /* private fields */ }Expand description
Counter index mask structure in SBI function calls for the PMU extension 11.
Implementations
Sourceimpl<T> CounterMask<T>where
T: SbiRegister,
impl<T> CounterMask<T>where
T: SbiRegister,
Sourcepub const IGNORE_MASK: T = T::FULL_MASK
pub const IGNORE_MASK: T = T::FULL_MASK
Special value to ignore the mask, and consider all bits as set.
Sourcepub const fn from_mask_base(
counter_idx_mask: T,
counter_idx_base: T,
) -> CounterMask<T>
pub const fn from_mask_base( counter_idx_mask: T, counter_idx_base: T, ) -> CounterMask<T>
Construct a CounterMask from mask value and base counter index.
Sourcepub const fn ignore_mask(&self) -> T
pub const fn ignore_mask(&self) -> T
Gets the special value for ignoring the mask parameter.
Sourcepub const fn into_inner(self) -> (T, T)
pub const fn into_inner(self) -> (T, T)
Returns mask and base parameters from the CounterMask.
Sourceimpl CounterMask
impl CounterMask
Sourcepub const fn has_bit(self, counter: usize) -> bool
pub const fn has_bit(self, counter: usize) -> bool
Returns whether the CounterMask contains the provided counter.